-- $Id: $
--
-- CalorieCounter.lua
--

-- TODO:
-- Hook tooltip to add nutritional info.
-- Add diets to nutritional info.

-----------------------------------------------------------------------
-- Localization

local L = {
   ["LABEL_TOTAL"] = "Total",
   ["LABEL_BURNED"] = "Burned",
   ["LABEL_GAINED"] = "Gained",
   ["CALORIE_TOOLTIP_FORMAT"] = "%d Calories",
   ["DOUBLEQUOTE_FORMAT"] = '"%s"',
}

local locale = GetLocale()

if locale == "enUS" then
   -- it's the default!
else
   -- unsupported locale
end
   

-----------------------------------------------------------------------
-- Addon Starts Here

local BMR_CAL_RATE = 84 / 3600 -- calories per hour
local RUNNING_CAL_RATE = 250 / 3600  -- 250 cal/hour
local FIGHTING_CAL_RATE = 800 / 3600 -- 800 cal/hour

CalorieCounter = {}
local CalorieCounter = CalorieCounter

-- Make localizations available to other files
CalorieCounter._L = L

-- local item_data = CalorieCounter.item_data
local item_data -- assigned in VARIABLES_LOADED

CalorieCounter.frame = CreateFrame("Frame", "CalorieCounter_EVENT_FRAME")

function CalorieCounter.OnEvent(frame, event, ...)
   return CalorieCounter[event](CalorieCounter, ...)
end

CalorieCounter.frame:SetScript("OnEvent", CalorieCounter.OnEvent)
CalorieCounter.frame:RegisterEvent("VARIABLES_LOADED")
CalorieCounter.frame:RegisterEvent("PLAYER_LOGIN")
CalorieCounter.frame:RegisterEvent("PLAYER_LOGOUT")

function CalorieCounter:VARIABLES_LOADED ()
   -- initialize
   item_data = self.item_data
   self:InitDB()
end

function CalorieCounter:PLAYER_LOGIN ()
   -- enable
   self.loginTime = GetTime()
   self:_InstallItemHooks()
   self:_InstallMovementHooks()
   self:_InstallTooltipHooks()
   self:_CreateFrames()
   self:RestorePosition()
   self.frame:RegisterEvent("PLAYER_REGEN_DISABLED")
   self.frame:RegisterEvent("PLAYER_REGEN_ENABLED")
end

function CalorieCounter:PLAYER_LOGOUT ()
   -- disable

   -- save burned/gained calories since they're calculated based on elapsed time
   self.profile.burned = self:GetBurnedCalories()
   self.profile.gained = self:GetGainedCalories()
   self.profile.gui_visible = self.gui:IsVisible()
end

-----------------------------------------------------------------------
-- Data

function CalorieCounter:InitDB()
   if type(CalorieData) ~= "table" then
      CalorieData = {}
   end

   self.db = CalorieData

   -- Store data per-char, but leave it in a per-account saved variable
   -- for potential new features
   local realm = GetRealmName()
   local character = UnitName("player")
   local profile = ("%s * %s"):format(realm, character)
  
   if type(self.db[profile]) ~= "table" then
      -- new profile
      self.db[profile] = {
	 gained = 0,
	 burned = 0,
      }
   end

   self.profile = self.db[profile]

   self:SetCalories(self.profile.gained, self.profile.burned)
end

-----------------------------------------------------------------------
-- Utility

local function printf(fmt, ...)
   ChatFrame1:AddMessage(tostring(fmt):format(...))
end

local function item_id_from_link (link)
   local id = link:match("|Hitem:(%d+):")
   return tonumber(id)
end

local itemlink_data_mt = {
   __index = function (t, k)
		local id = item_id_from_link(k)
		local data = item_data[id] or false
		rawset(t, k, data)
		return data
	     end,
}
local itemlink_data = setmetatable({}, itemlink_data_mt)

-----------------------------------------------------------------------
-- Calorie Monitoring

local total_gained = 0
local total_burned = 0

function CalorieCounter:SetCalories (gained, burned)
   total_gained = gained
   total_burned = burned
end

function CalorieCounter:GainedCalories (calories, message)
   -- printf("Gained %f calories: %s", calories, message)
   total_gained = total_gained + calories
   self:CheckHunger()
end

function CalorieCounter:BurnedCalories (calories, message)
   -- printf("Burned %f calories: %s", calories, message)
   total_burned = total_burned + calories
   self:CheckHunger()
end

function CalorieCounter:GetGainedCalories ()
   return total_gained
end

function CalorieCounter:GetBurnedCalories (now)
   if not now then
      now = GetTime()
   end

   local elapsed = now - self.loginTime
   local bmr_burned = elapsed * BMR_CAL_RATE

   local running_time = now - (self.movementStartTime or now)
   local temp_running_calories = running_time * RUNNING_CAL_RATE

   local fighting_time = now - (self.combatStartTime or now)
   local temp_fighting_calories = fighting_time * FIGHTING_CAL_RATE

   local temp_burned = temp_running_calories + temp_fighting_calories

   return total_burned + bmr_burned + temp_burned
end

function CalorieCounter:GetTotalCalories ()
   local burned = self:GetBurnedCalories()
   local gained = self:GetGainedCalories()
   return gained - burned
end

local fat_emotes = {
   'BURP',
   'FART',
   'MOO',
}

local skinny_emotes = {
   'DROOL',
   'HUNGRY',
}

local function DoRandomEmote (emotes)
   DoEmote(emotes[math.random(#emotes)])
end

function CalorieCounter:CheckHunger ()
   local now = GetTime()
   local elapsed = now - self.loginTime
   local calories = self:GetTotalCalories()
   local hours = elapsed / 3600
   local since_last_emote = now - (self.emoteTime or 0)

   if hours < 0.5 or since_last_emote < 1800 then
   -- if since_last_emote < 60 then
      -- give the player some time to do things before deciding if they're
      -- hungry or fat
      -- also, wait 30 min between emotes (1800 = 30 * 60)
      return
   end

   local cal_per_hour = calories / hours
   if cal_per_hour > 100 then
      -- fatty
      DoRandomEmote(fat_emotes)
      self.emoteTime = now
   elseif cal_per_hour < -100 then
      -- skinny
      DoRandomEmote(skinny_emotes)
      self.emoteTime = now
   end
end
   

-----------------------------------------------------------------------
-- Item Usage

function CalorieCounter:_InstallItemHooks ()
   -- local self = self
   local function UseContainerItem_Hook(bagId, slot, onSelf)
      item = GetContainerItemLink(bagId, slot)
      self:ItemUsed(item)
   end
   hooksecurefunc("UseContainerItem", UseContainerItem_Hook)

   function UseAction_Hook (slot, checkCursor, onSelf)
      local atype, id, subtype = GetActionInfo(slot)
      if atype == "item" then
	 self:ItemUsed(id)
      end
   end
   hooksecurefunc("UseAction", UseAction_Hook)

   function UseItemByName_Hook (name, target)
      self:ItemUsed(name)
   end
   hooksecurefunc("UseItemByName", UseItemByName_Hook)
end

function CalorieCounter:ItemUsed (item)
   local _, link = GetItemInfo(item)
   local id = item_id_from_link(link)

   local startTime, duration, enable = GetItemCooldown(id)
   -- if startTime is not zero then the item is still on cooldown
   if startTime == 0 and IsUsableItem(id) and IsConsumableItem(id) then
      if self.item_data[id] then
	 local data = self.item_data[id]
	 local calories = data['calories']
	 local note_eat = data['note_eat']
	 if type(calories) == "number" then
	    self:GainedCalories(calories, ("Consumed %s"):format(link))
	    if note_eat then
	       printf(note_eat)
	    end
	 else
	    printf("Unknown item: %s (%d)", link, id)
	 end
      end
   end
end

-----------------------------------------------------------------------
-- Combat

function CalorieCounter:PLAYER_REGEN_DISABLED ()
   self.combatStartTime = GetTime()
end

function CalorieCounter:PLAYER_REGEN_ENABLED ()
   local elapsed = GetTime() - self.combatStartTime
   local calories = elapsed * FIGHTING_CAL_RATE
   self:BurnedCalories(calories, ("Spent %f sec. in combat."):format(elapsed))
end

-----------------------------------------------------------------------
-- Walking / Running

local moving_forward = false
local moving_backward = false
local strafing_left = false
local strafing_right = false
local auto_run = false
local mouse_running = false
local worldframe_lmb = false -- CameraOrSelectOrMoveStart/Stop
local worldframe_rmb = false -- TurnOrActionStart/Stop

local function start_moving ()
   if IsMounted() then
      -- it only coungs if you're on foot!
      return
   end
   if not CalorieCounter.movementStartTime then
      CalorieCounter.movementStartTime = GetTime()
   end
end

local function stop_moving (force)
   if not force and (moving_forward or moving_backward or
		     strafing_left or strafing_right or
		     mouse_running or auto_run) then
      -- still moving
      return
   end

   if not CalorieCounter.movementStartTime then
      -- not moving
      return
   end

   local elapsed = GetTime() - CalorieCounter.movementStartTime
   CalorieCounter.movementStartTime = nil
   local calories = elapsed * RUNNING_CAL_RATE
   CalorieCounter:BurnedCalories(calories,
				 ("Spent %f sec. running."):format(elapsed))
end

local function check_mouse_running ()
   if worldframe_lmb and worldframe_rmb then
      mouse_running = true
      auto_run = false
      start_moving()
   else
      mouse_running = false
      stop_moving()
   end
end

function CalorieCounter:_InstallMovementHooks ()
   local function MoveForwardStart_Hook ()
      moving_forward = true
      auto_run = false
      start_moving()
   end
   hooksecurefunc("MoveForwardStart", MoveForwardStart_Hook)

   local function MoveForwardStop_Hook ()
      moving_forward = false
      stop_moving()
   end
   hooksecurefunc("MoveForwardStop", MoveForwardStop_Hook)

   local function MoveBackwardStart_Hook ()
      moving_backward = true
      auto_run = false
      start_moving()
   end
   hooksecurefunc("MoveBackwardStart", MoveBackwardStart_Hook)

   local function MoveBackwardStop_Hook ()
      moving_backward = false
      -- turning on auto-run while holding down the move backwards key
      -- causes the player to stop moving until the move backwards key is
      -- released
      if auto_run then
	 start_moving()
      else
	 stop_moving()
      end
   end
   hooksecurefunc("MoveBackwardStop", MoveBackwardStop_Hook)

   local function StrafeLeftStart_Hook ()
      strafing_left = true
      start_moving()
   end
   hooksecurefunc("StrafeLeftStart", StrafeLeftStart_Hook)

   local function StrafeLeftStop_Hook ()
      strafing_left = false
      stop_moving()
   end
   hooksecurefunc("StrafeLeftStop", StrafeLeftStop_Hook)

   local function StrafeRightStart_Hook ()
      strafing_right = true
      start_moving()
   end
   hooksecurefunc("StrafeRightStart", StrafeRightStart_Hook)

   local function StrafeRightStop_Hook ()
      strafing_right = false
      stop_moving()
   end
   hooksecurefunc("StrafeRightStop", StrafeRightStop_Hook)

   local function ToggleAutoRun_Hook ()
      auto_run = not auto_run
      if auto_run then
	 if moving_backward and not strafing_right and not strafing_left then
	    -- turning on auto-run while holding down the move backwards key
	    -- causes the player to stop moving until the move backwards key is
	    -- released
	    stop_moving(true)
	 else
	    start_moving()
	 end
      else
	 stop_moving()
      end
   end
   hooksecurefunc("ToggleAutoRun", ToggleAutoRun_Hook)

   local function CameraOrSelectOrMoveStart_Hook ()
      worldframe_lmb = true
      check_mouse_running()
   end
   hooksecurefunc("CameraOrSelectOrMoveStart", CameraOrSelectOrMoveStart_Hook)

   local function CameraOrSelectOrMoveStop_Hook ()
      worldframe_lmb = false
      check_mouse_running()
   end
   hooksecurefunc("CameraOrSelectOrMoveStop", CameraOrSelectOrMoveStop_Hook)

   local function TurnOrActionStart_Hook ()
      worldframe_rmb = true
      check_mouse_running()
   end
   hooksecurefunc("TurnOrActionStart", TurnOrActionStart_Hook)

   local function TurnOrActionStop_Hook ()
      worldframe_rmb = false
      check_mouse_running()
   end
   hooksecurefunc("TurnOrActionStop", TurnOrActionStop_Hook)
end

-----------------------------------------------------------------------
-- Tooltip

function CalorieCounter:_InstallTooltipHooks ()
   -- this will get called very often, optimize this!
   local function GameTooltip__OnTooltipSetItem_Hook (tooltip, ...)
      local _, itemlink = tooltip:GetItem()
      if IsConsumableItem(itemlink) then
	 local data = itemlink_data[itemlink]
	 if data ~= false then
	    local calories = data['calories']
	    local note_tooltip = data['note_tooltip']
	    local calories_txt = L["CALORIE_TOOLTIP_FORMAT"]:format(calories)

	    tooltip:AddLine(' ')
	    tooltip:AddLine(calories_txt)
	    if note_tooltip then
	       local tooltip_txt = L["DOUBLEQUOTE_FORMAT"]:format(note_tooltip)
	       tooltip:AddLine(tooltip_txt)
	    end
	 end
      end
   end

   GameTooltip:HookScript("OnTooltipSetItem",
			  GameTooltip__OnTooltipSetItem_Hook)
end

-----------------------------------------------------------------------
-- GUI

function CalorieCounter.GUI_OnUpdate (frame, elapsed)
   local throttle = frame.throttle or 0
   throttle = throttle - elapsed
   if throttle <= 0 then
      CalorieCounter:GUI_Update()
      throttle = 1
   end
   frame.throttle = throttle
end

function CalorieCounter:GUI_Update ()
   local mode = self.gui_mode or "total"
   local calories = self:GetTotalCalories()
   if mode == "total" then
      calories = self:GetTotalCalories() 
   elseif mode == "gained" then
      calories = self:GetGainedCalories()
   elseif mode == "burned" then
      calories = self:GetBurnedCalories()
   end
   if calories then
      self:SetGuiText(("%8.2f"):format(calories))
   else
      self:SetGuiText("ERROR")
   end
end

function CalorieCounter.GUI_Common_OnEnter (frame)
   local self = CalorieCounter
   self.gui.tab:Show()
end

function CalorieCounter.GUI_Common_OnLeave (frame)
   local self = CalorieCounter
   local focus = GetMouseFocus()
   if not self.is_moving and
      focus ~= self.gui and focus ~= self.gui.tab and
      focus ~= self.gui.modeButton and focus ~= self.gui.closeButton then
      self.gui.tab:Hide()
   end
end

function CalorieCounter.GUI_Common_OnMouseDown (frame, button)
   local self = CalorieCounter
   if button == "LeftButton" then
      self.gui:StartMoving()
      self:RestorePosition()
      self.is_moving = true
   end
end

function CalorieCounter.GUI_Common_OnMouseUp (frame)
   local self = CalorieCounter
   if self.is_moving then
      self.gui:StopMovingOrSizing()
      self:SavePosition()
      self.is_moving = false
      -- sometimes you can drag the mouse outside the frame
      self.GUI_Common_OnLeave(frame)
   end
end

function CalorieCounter.GUI_ModeButton_OnClick (frame)
   local self = CalorieCounter
   local mode = self.gui_mode or "total"
   local label
   if mode == "total" then
      mode = "gained"
      label = L["LABEL_GAINED"]
   elseif mode == "gained" then
      mode = "burned"
      label = L["LABEL_BURNED"]
   elseif mode == "burned" then
      mode = "total"
      label = L["LABEL_TOTAL"]
   end

   self.gui_mode = mode
   self.gui.label:SetText(label)
   self:GUI_Update()
end

function CalorieCounter.GUI_CloseButton_OnClick (frame)
   local self = CalorieCounter
   self.gui:Hide()
   self.gui.tab:Hide()
end

function CalorieCounter:_CreateFrames ()
   local width = 150
   local left_width = 128
   local right_width = 10
   local middle_width = width - left_width - right_width + 8
   local tab_width = width - 4
   local tab_side_width = 16
   local tab_middle_width = tab_width - tab_side_width * 2
   local tab_height = 18
   local tab_alpha = 0.9

   -- Main Frame
   local f = CreateFrame("Frame", "CalorieCounter_GUI", UIParent)
   f:Hide()
   f:SetWidth(width)
   f:SetHeight(28)
   f:SetMovable(true)
   f:SetPoint("TOPLEFT", UIParent, "TOPLEFT",
	      UIParent:GetWidth() / 2, 0 - UIParent:GetHeight() / 2)
   f:EnableMouse(true)
   f:RegisterForDrag("LeftButton")
   f:SetScript("OnUpdate", CalorieCounter.GUI_OnUpdate)
   f:SetScript("OnMouseDown", CalorieCounter.GUI_Common_OnMouseDown)
   f:SetScript("OnMouseUp", CalorieCounter.GUI_Common_OnMouseUp)
   f:SetScript("OnEnter", CalorieCounter.GUI_Common_OnEnter)
   f:SetScript("OnLeave", CalorieCounter.GUI_Common_OnLeave)
   self.gui = f

   -- Background
   f.bgLeft = f:CreateTexture("CalorieCounter_GUI_BgLeft", "BACKGROUND")
   f.bgLeft:SetTexture("Interface\\TimeManager\\TimerBackground")
   f.bgLeft:SetTexCoord(0, 1.0, 0, 0.453125)
   f.bgLeft:SetWidth(left_width)
   f.bgLeft:SetHeight(29)
   f.bgLeft:SetPoint("LEFT", f, "LEFT", -8, 0)

   f.bgMiddle = f:CreateTexture("CalorieCounter_GUI_BgMiddle", "BACKGROUND")
   f.bgMiddle:SetTexture("Interface\\TimeManager\\TimerBackground")
   f.bgMiddle:SetTexCoord(0.75, 1.0, 0, 0.453125)
   f.bgMiddle:SetWidth(middle_width)
   f.bgMiddle:SetHeight(29)
   f.bgMiddle:SetPoint("TOPLEFT", f.bgLeft, "TOPRIGHT", 0, 0)
   
   f.bgRight = f:CreateTexture("CalorieCounter_GUI_BgRight", "BACKGROUND")
   f.bgRight:SetTexture("Interface\\TimeManager\\TimerBackground")
   f.bgRight:SetTexCoord(0, 0.078125, 0.453125, 1)
   f.bgRight:SetWidth(right_width)
   f.bgRight:SetHeight(35)
   f.bgRight:SetPoint("TOPLEFT", f.bgMiddle, "TOPRIGHT", 0, 1)

   -- Numeric Display
   f.text = f:CreateFontString("CalorieCounter_GUI_Text", "BACKGROUND",
			       "GameFontHighlightLarge")
   f.text:SetPoint("RIGHT", f, "RIGHT", -44 - middle_width - right_width, 0)
   f.text:SetText("99999")

   -- Label for Mode Button
   f.label = f:CreateFontString("CalorieCounter_GUI_Label", "BACKGROUND",
				"GameFontHighlightLarge")
   f.label:SetText(L["LABEL_TOTAL"])

   -- Mode Button
   f.modeButton = CreateFrame("Button", "CalorieCounter_GUI_ModeButton", f,
			      "UIPanelButtonTemplate")
   f.modeButton:SetWidth( 30 + middle_width + right_width ) -- 52
   f.modeButton:SetHeight(22)
   f.modeButton:SetFontString(f.label)
   f.modeButton:SetPoint("RIGHT", f, "RIGHT", -4, 0)
   f.modeButton:SetScript("OnClick", CalorieCounter.GUI_ModeButton_OnClick)
   f.modeButton:SetScript("OnEnter", CalorieCounter.GUI_Common_OnEnter)
   f.modeButton:SetScript("OnLeave", CalorieCounter.GUI_Common_OnLeave)


   -- Handle/Tab
   f.tab = CreateFrame("Frame", "CalorieCounter_GUI_Tab", f)
   f.tab:SetWidth(tab_width)
   f.tab:SetHeight(tab_height)
   f.tab:EnableMouse(true)
   f.tab:RegisterForDrag("LeftButton")
   f.tab:SetPoint("BOTTOM", f, "TOP", 0, 0)
   f.tab:SetScript("OnMouseDown", CalorieCounter.GUI_Common_OnMouseDown)
   f.tab:SetScript("OnMouseUp", CalorieCounter.GUI_Common_OnMouseUp)
   f.tab:SetScript("OnEnter", CalorieCounter.GUI_Common_OnEnter)
   f.tab:SetScript("OnLeave", CalorieCounter.GUI_Common_OnLeave)
   f.tab:Hide()

   -- Handle/Tab Background
   f.tabBgLeft = f.tab:CreateTexture("CalorieCounter_GUI_Tab_BgLeft",
				     "BACKGROUND")
   f.tabBgLeft:SetTexture("Interface\\ChatFrame\\ChatFrameTab")
   f.tabBgLeft:SetTexCoord(0, 0.25, 0, 1)
   f.tabBgLeft:SetAlpha(tab_alpha)
   f.tabBgLeft:SetWidth(tab_side_width)
   f.tabBgLeft:SetHeight(tab_height + 5)
   f.tabBgLeft:SetPoint("BOTTOMLEFT", f.tab, "BOTTOMLEFT", 0, 0)

   f.tabBgMiddle = f.tab:CreateTexture("CalorieCounter_GUI_Tab_BgMiddle",
				       "BACKGROUND")
   f.tabBgMiddle:SetTexture("Interface\\ChatFrame\\ChatFrameTab")
   f.tabBgMiddle:SetTexCoord(0.25, 0.75, 0, 1)
   f.tabBgMiddle:SetAlpha(tab_alpha)
   f.tabBgMiddle:SetWidth(tab_middle_width)
   f.tabBgMiddle:SetHeight(tab_height + 5)
   f.tabBgMiddle:SetPoint("LEFT", f.tabBgLeft, "RIGHT", 0, 0)

   f.tabBgRight = f.tab:CreateTexture("CalorieCounter_GUI_Tab_BgRight",
				      "BACKGROUND")
   f.tabBgRight:SetTexture("Interface\\ChatFrame\\ChatFrameTab")
   f.tabBgRight:SetTexCoord(0.75, 1, 0, 1)
   f.tabBgRight:SetAlpha(tab_alpha)
   f.tabBgRight:SetWidth(tab_side_width)
   f.tabBgRight:SetHeight(tab_height + 5)
   f.tabBgRight:SetPoint("LEFT", f.tabBgMiddle, "RIGHT", 0, 0)

   -- Tab Label
   f.tabText = f.tab:CreateFontString("CalorieCounter_GUI_Tab_Text",
				      "BACKGROUND", "GameFontNormalSmall")
   f.tabText:SetText("CalorieCounter")
   f.tabText:SetPoint("TOP", f.tab, "TOP", 0, -5)

   -- Close Button
   f.closeButton = CreateFrame("Button", "CalorieCounter_GUI_CloseButton",
			       f.tab, "UIPanelCloseButton")
   f.closeButton:SetWidth(20)
   f.closeButton:SetHeight(20)
   f.closeButton:SetPoint("TOPRIGHT", f.tab, "TOPRIGHT", -2, -1)
   f.closeButton:SetScript("OnClick", CalorieCounter.GUI_CloseButton_OnClick)
   f.closeButton:SetScript("OnEnter", CalorieCounter.GUI_Common_OnEnter)
   f.closeButton:SetScript("OnLeave", CalorieCounter.GUI_Common_OnLeave)
end

function CalorieCounter:SetGuiText (value)
   self.gui.text:SetText(value)
end

function CalorieCounter:SavePosition ()
   local f = self.gui
   local s = f:GetEffectiveScale()
   local uiScale = UIParent:GetEffectiveScale()
   local anchor = "TOPLEFT"
   local x, y

   x = f:GetLeft() * s
   y = f:GetTop() * s - UIParent:GetHeight() * uiScale

   self.profile.gui_x = x
   self.profile.gui_y = y
end

function CalorieCounter:RestorePosition ()
   local f = self.gui
   local s = f:GetEffectiveScale()
   local x = self.profile.gui_x or UIParent:GetWidth() / 2
   local y = self.profile.gui_y or 0 - UIParent:GetHeight() / 2

   x, y = x/s, y/s
   f:ClearAllPoints()
   f:SetPoint("TOPLEFT", UIParent, "TOPLEFT", x, y)

   if self.profile.gui_visible then
      f:Show()
   end
end

function CalorieCounter:ResetPosition ()
   -- relative to BOTTOMLEFT
   local cursor_x, cursor_y = GetCursorPosition()
   
   local new_x = cursor_x
   local new_y = cursor_y - UIParent:GetHeight() * UIParent:GetEffectiveScale()

   self.profile.gui_x = new_x
   self.profile.gui_y = new_y
   self.profile.gui_visible = true
   self:RestorePosition()
end

-----------------------------------------------------------------------
-- Slash Commands

local function slash_command (msg)
   local self = CalorieCounter

   if msg == "resetpos" then
      self:ResetPosition()
   elseif msg == "show" then
      self.gui:Show()
   elseif msg == "hide" then
      self.gui:Hide()
   elseif msg == "reset" then
      self:SetCalories(0, 0)
   else
      local gui = self.gui
      if gui:IsVisible() then
	 gui:Hide()
      else
	 gui:Show()
      end
   end
end

SLASH_CALORIE1 = "/calorie"
SlashCmdList["CALORIE"] = slash_command
